package com.clx.performance.controller.pc.linewarn;


import com.baomidou.mybatisplus.core.metadata.IPage;
import com.clx.performance.enums.linewarn.LineWarnConfigEnum;
import com.clx.performance.param.pc.linewarn.*;
import com.clx.performance.service.linewarn.LineWarnConfigService;
import com.clx.performance.vo.pc.linewarn.LineWarnConfigVO;
import com.msl.common.base.PageData;
import com.msl.common.enums.ResultCodeEnum;
import com.msl.common.exception.ServiceSystemException;
import com.msl.common.result.Result;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.validation.constraints.NotNull;
import java.util.List;
import java.util.Objects;


@Slf4j
@RestController
@Validated
@Api(tags = "线路预警-预警配置")
@RequestMapping(value="/pc/lineWarnConfig")
public class LineWarnConfigController {

    @Autowired
    private LineWarnConfigService lineWarnConfigService;

    @ApiOperation(value = "新增配置", notes = "<br>By:艾庆国")
    @PostMapping("/saveConfig")
    public Result<Void> saveConfig(@RequestBody @Validated LineWarnConfigAddParam param) {

        if (Objects.equals(param.getWarnType(), LineWarnConfigEnum.WarnType.TAKE.getCode())
                || Objects.equals(param.getWarnType(), LineWarnConfigEnum.WarnType.LOAD.getCode())
                || Objects.equals(param.getWarnType(), LineWarnConfigEnum.WarnType.UNLOAD.getCode())
        ){
            if (Objects.isNull(param.getTimeRatio())){
                throw new ServiceSystemException(ResultCodeEnum.ILLEGAL_PARAMETER, "时间占比不能为空");}
            if (Objects.isNull(param.getTaskCompleteRatio())){
                throw new ServiceSystemException(ResultCodeEnum.ILLEGAL_PARAMETER, "任务完成占比不能为空");}
        }
        if (Objects.equals(param.getWarnType(), LineWarnConfigEnum.WarnType.CONGESTION.getCode())
        ){
            if (Objects.isNull(param.getCongestionTime())){
                throw new ServiceSystemException(ResultCodeEnum.ILLEGAL_PARAMETER, "交通增加时间不能为空");}
        }

        lineWarnConfigService.saveConfig(param);

        return Result.ok();
    }

    @ApiOperation(value = "更新配置", notes = "<br>By:艾庆国")
    @PostMapping("/updateConfig")
    public Result<Void> updateConfig(@RequestBody @Validated LineWarnConfigUpdateParam param) {
        if (Objects.equals(param.getWarnType(), LineWarnConfigEnum.WarnType.TAKE.getCode())
                || Objects.equals(param.getWarnType(), LineWarnConfigEnum.WarnType.LOAD.getCode())
                || Objects.equals(param.getWarnType(), LineWarnConfigEnum.WarnType.UNLOAD.getCode())
        ){
            if (Objects.isNull(param.getTimeRatio())){
                throw new ServiceSystemException(ResultCodeEnum.ILLEGAL_PARAMETER, "时间占比不能为空");}
            if (Objects.isNull(param.getTaskCompleteRatio())){
                throw new ServiceSystemException(ResultCodeEnum.ILLEGAL_PARAMETER, "任务完成占比不能为空");}
        }
        if (Objects.equals(param.getWarnType(), LineWarnConfigEnum.WarnType.CONGESTION.getCode())
        ){
            if (Objects.isNull(param.getCongestionTime())){
                throw new ServiceSystemException(ResultCodeEnum.ILLEGAL_PARAMETER, "交通增加时间不能为空");}
        }
        lineWarnConfigService.updateConfig(param);
        return Result.ok();
    }

    @ApiOperation(value = "更新配置状态", notes = "<br>By:艾庆国")
    @PostMapping("/updateStatus")
    public Result<Void> updateStatus(@RequestBody @Validated LineWarnConfigStatusUpdateParam param) {

        lineWarnConfigService.updateStatus(param);
        return Result.ok();
    }

    @ApiOperation(value = "删除配置", notes = "<br>By:艾庆国")
    @PostMapping("/deleteConfig")
    public Result<Void> deleteConfig(@RequestBody @Validated LineWarnConfigDeleteUpdateParam param) {
        lineWarnConfigService.deleteConfig(param);
        return Result.ok();
    }

    @ApiOperation(value = "获取配置信息",notes = "<br>By:艾庆国")
    @GetMapping("/getConfig")
    public Result<LineWarnConfigVO> getConfig(@NotNull(message = "id不能为空") @Validated Integer id) {
        return Result.ok(lineWarnConfigService.getConfig(id));
    }

    @ApiOperation(value = "分页搜索配置列表",notes = "<br>By:艾庆国")
    @PostMapping("/pageConfig")
    public Result<PageData<LineWarnConfigVO>> pageConfig(@RequestBody @Validated PageLineWarnConfigParam param) {
        IPage<LineWarnConfigVO> page = lineWarnConfigService.pageConfig(param);
        return Result.page(page.getRecords(), page.getTotal(), page.getPages());
    }

    @ApiOperation(value = "预警类型下拉",notes = "<br>By:艾庆国")
    @GetMapping("/listWarnType")
    public Result<List<LineWarnConfigVO.WarnTypeItem>> listWarnType() {
        return Result.ok(lineWarnConfigService.listWarnType());
    }

}
